@@ -1,41 +1,41 @@
 import sys
-# global ONE_LETTER
-# global THREE_LETTER
-# global FULL_NAME
-from types import FunctionType
-from apalib.config import *
+import apalib.config as config
 global FLAGS
 FLAGS = {}
 
 
 class AminoAcid:
-    def __init__(self, **kwargs):
-        self.number = None
-        self.atoms = None
-        self.name = None
-        self.rotamer = None
-        self.vector = None
-        self.heptad = None
-        self.centroid = None
-        set_name = True
-        if 'set_name' in kwargs:
-            set_name = kwargs['set_name']
-        if 'name' in kwargs:
-            self.SetName(kwargs['name'], set_name)
-        if 'number' in kwargs:
-            self.SetNumber(kwargs['number'])
-        if 'atoms' in kwargs:
-            self.SetAtoms(kwargs['atoms'])
+    def __init__(self, seqNum = None, atoms = None, resName = None, rotamer = None, vector = None, set_name = True,
+                 heptad = None, centroid = None, chainID=None):
+        self.rotamer = rotamer
+        self.vector = vector
+        self.heptad = heptad
+        self.centroid = centroid
+        self.SetResName(resName, set_name)
+        self.SetSeqNum(seqNum)
+        self.SetAtoms(atoms)
+        self.SetChainID(chainID)
+
+    def DeepCopy(self):
+        return AminoAcid(number=self.seqNum,
+                         atoms=self.atoms,
+                         name=self.resName,
+                         rotamer=self.rotamer,
+                         vector=self.vector,
+                         heptad=self.heptad,
+                         centroid=self.centroid,
+                         chainID=self.chainID,
+                         set_name=True)
 
     def AddAttribute(self, attr, var):
         self.__dict__[attr] = var
 
-    def SetNumber(self, num):
-        self.number = num
+    def SetSeqNum(self, num):
+        self.seqNum = num
 
     def SetAtoms(self, atoms):
         self.atoms = atoms
-        self.CalculateCentroid(atoms)
+        # self.CalculateCentroid()
 
     def InsertAtom(self, atom):
         if self.atoms is None:
@@ -73,53 +73,55 @@ class AminoAcid:
                 lst += retDict['Common']
         return retDict
 
-
-
-
-
-
     def GetCA(self):
         if self.atoms is None:
             return None
         for atom in self.atoms:
-            if atom.GetID() == 'CA':
+            if atom.GetName() == 'CA':
                 return atom
         return None
 
-    def SetName(self, name, set_name):
+    def SetResName(self, name, set_name):
+        if name is None:
+            self.resName = name
+            return
         if not set_name:
-            self.name = name
+            self.resName = name
             self.RaiseFlag('NO_NAME_CHECK')
             return
         else:
             self.ClearFlag('NO_NAME_CHECK')
         if len(name) == 3 and name in THREE_LETTER:
-            self.name = name
+            self.resName = name
         elif len(name) == 3 and name not in THREE_LETTER:
-            self.name = None
+            self.resName = None
         elif len(name) == 1:
-            self.name = self.OneToThree(name)
+            self.resName = self.OneToThree(name)
         elif len(name) <= 4:
             if name[-3:] in THREE_LETTER:
-                self.name = name[-3:]
+                self.resName = name[-3:]
                 self.rotamer = name[:-3]
             else:
-                self.name = None
+                self.resName = None
         elif len(name) == 2:
-            self.name = None
-        if self.name is not None:
+            self.resName = None
+        if self.resName is not None:
             self.RaiseFlag('BAD_NAME')
         else:
             self.RaiseFlag('BAD_NAME')
-        if self.name is not None and self.rotamer is not None:
+        if self.resName is not None and self.rotamer is not None:
             self.RaiseFlag('MARKED')
-        elif self.name is not None and self.rotamer is None:
+        elif self.resName is not None and self.rotamer is None:
             self.ClearFlag('MARKED')
 
     def SetHeptad(self, heptad):
         self.heptad = heptad
 
-    def CalculateCentroid(self, atoms):
+    def CalculateCentroid(self):
+        if 'atoms' not in self.__dict__:
+            self.centroid = None
+            return
+        #TODO switch to using the JSON for this
         # *For Glycine, only the alpha carbon is considered
         # *For Alanine, only the beta carbon is considered
         AAs = {
@@ -146,13 +148,13 @@ class AminoAcid:
             "TRP": ['CG', 'CD1', 'CD2', 'NE1', 'CE2', 'CE3', 'CZ2', 'CZ3', 'CH2']
         }
         # TODO Deal with non-1 occupancy
-        if len(self.name) == 3 and self.name in AAs:
+        if len(self.resName) == 3 and self.resName in AAs:
             # If residue is not specified
             x_coord = 0
             y_coord = 0
             z_coord = 0
             num_atoms = 0
-            for atom in [atom for atom in atoms if atom.id in AAs[self.name]]:
+            for atom in [atom for atom in self.atoms if atom.name in AAs[self.resName]]:
                 if atom.element == 'H':
                     continue
                 x_coord += atom.GetCoordinates()[0]
@@ -166,8 +168,8 @@ class AminoAcid:
                 self.centroid.append(z_coord / num_atoms)
             except ZeroDivisionError:
                 self.centroid.clear()
-                beta = [atom for atom in atoms if atom.id == 'CB']
-                alpha = [atom for atom in atoms if atom.id == 'CA']
+                beta = [atom for atom in self.atoms if atom.name == 'CB']
+                alpha = [atom for atom in self.atoms if atom.name == 'CA']
                 if len(beta) != 0:
                     self.centroid.append(beta[0].coordinates[0])
                     self.centroid.append(beta[0].coordinates[1])
@@ -188,7 +190,7 @@ class AminoAcid:
                            self.centroid[1] - self.GetCA().GetCoordinates()[1],
                            self.centroid[2] - self.GetCA().GetCoordinates()[2]]
 
-        elif len(self.name) == 4 and self.name[1:] in AAs:
+        elif len(self.resName) == 4 and self.resName[1:] in AAs:
             sys.stderr.write("ROTAMER!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!")
         else:
             sys.exit("UNKNOWN AMINO ACID YO. DO SOMETHING ABOUT THIS!!!!!")
@@ -196,6 +198,23 @@ class AminoAcid:
     def GetCentroid(self):
         return self.centroid
 
+    def GetVector(self):
+        return self.vector
+
+    def SetChainID(self, c):
+        self.chainID = c
+
+    def GetASA(self, form):
+        d = config.data
+        name = config.data.Map("Amino Acids", config.data.Standardize(self.resName))
+        return config.data.GetJson()["Amino Acids"][name]["ASA"][form]
+
+    def WriteForPDB(self):
+        retstr = ""
+        for atom in self.atoms:
+            retstr += atom.WritePDB(intro="ATOM  ")
+        return retstr
+
     @staticmethod
     def OneToThree(oln):
         if oln in ONE_LETTER:
@@ -243,13 +262,13 @@ class AminoAcid:
         return
 
     def __lt__(self, other):
-        return self.number < other.number
+        return self.seqNum < other.number
 
     def __repr__(self):
-        return f"RESIDUE: {self.name}, NUMBER: {self.number}"
+        return f"RESIDUE: {self.resName}, NUMBER: {self.seqNum}"
 
     def __str__(self):
-        return f"{self.name} {self.number}"
+        return f"{self.resName} {self.seqNum}"
 
 
 # Shoved down here for cleanliness
